import 'dart:async';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

const MethodChannel _channel = const MethodChannel('passbase_flutter');

class PassbaseFlutter {
  static Future<void> initialize(
      {@required String apiKey,
      Map<String, String> additionalAttributes,
      String prefillUserEmail}) async {
    assert(apiKey != null, "API key cannot be null!");

    Map arguments = new Map();
    arguments["apiKey"] = apiKey;

    if (additionalAttributes != null) {
      arguments["additionalAttributes"] = additionalAttributes;
    }
    if (prefillUserEmail != null) {
      arguments["prefillUserEmail"] = prefillUserEmail;
    }

    await _channel.invokeMethod('initialize', arguments);
    return;
  }

  static Future<void> setTestMode(bool value) async {
    assert(value != null, "Calue cannot be null!");
    if (Platform.isAndroid) {
      await _channel.invokeMethod('setTestMode', value);
    }
    return;
  }

  static Future<void> startVerification({
    Function onVerificationStart,
    Function(String) onVerificationSuccess,
    Function onVerificationCancel,
  }) async {
    _channel.setMethodCallHandler((call) {
      if (call.method == "verification_cancel") {
        onVerificationCancel();
      } else if (call.method == "verification_complete") {
        onVerificationSuccess(call.arguments.toString());
      } else if (call.method == "verification_start") {
        onVerificationStart();
      }
      return;
    });

    await _channel.invokeMethod('startVerification');
  }
}

class PassbaseButton extends StatelessWidget {
  final double width;
  final double height;
  final Function(String) onVerificationSuccess;
  final Function onVerificationCancel;
  final Function onVerificationStart;

  PassbaseButton(
      {this.width,
      this.height,
      this.onVerificationSuccess,
      this.onVerificationCancel,
      this.onVerificationStart}) {
    _channel.setMethodCallHandler((call) {
      if (call.method == "verification_cancel") {
        onVerificationCancel();
      } else if (call.method == "verification_complete") {
        onVerificationSuccess(call.arguments.toString());
      } else if (call.method == "verification_start") {
        onVerificationStart();
      }
      return;
    });
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: width,
      height: height,
      child: Platform.isAndroid
          ? AndroidView(
              viewType: "PassbaseButton",
            )
          : UiKitView(
              viewType: "PassbaseButton",
            ),
    );
  }
}
